@@ -29,15 +29,12 @@ except ImportError:     # Python 2
     int_types = int, long
 
 try:
-    # first, try importing directly
     from ast import literal_eval
-except ImportError:
+except ImportError:     # Python 2.5
     import _ast
 
     # Port of Python 2.6's ast.literal_eval for use under Python 2.5
-    SAFE_CONSTANTS = {'None': None, 'True': True, 'False': False}
-
-    def _convert(node):
+    def _convert(node, _consts={'None': None, 'True': True, 'False': False}):
         if isinstance(node, _ast.Str):
             return node.s
         if isinstance(node, _ast.Num):
@@ -47,10 +44,10 @@ except ImportError:
         if isinstance(node, _ast.List):
             return list(map(_convert, node.elts))
         if isinstance(node, _ast.Dict):
-            return dict((_convert(k), _convert(v))
-                        for (k, v) in zip(node.keys, node.values))
-        if isinstance(node, _ast.Name) and node.id in SAFE_CONSTANTS:
-            return SAFE_CONSTANTS[node.id]
+            return dict([(_convert(k), _convert(v))
+                         for (k, v) in zip(node.keys, node.values)])
+        if isinstance(node, _ast.Name) and node.id in _consts:
+            return _consts[node.id]
         raise ValueError('malformed or disallowed expression')
 
     def literal_eval(node_or_string):
@@ -62,7 +59,7 @@ except ImportError:
         return _convert(node_or_string)
 
 
-__version__ = '1.4.3'
+__version__ = '1.4.4'
 __all__ = ['Client', 'Model', 'Record', 'RecordList', 'Service',
            'format_exception', 'read_config', 'start_openerp_services']
 
@@ -130,11 +127,8 @@ _methods_6_1 = {
                'authenticate', 'get_os_time', 'get_sqlcount'],
     'object': ['execute_kw'],
     'report': ['render_report'],
-    'wizard': [],
 }
 # Hidden methods:
-#  - (not in 6.1) 'common': ['logout', 'ir_get', 'ir_set', 'ir_del']
-#  - (not in 6.1) 'object': ['obj_list']
 #  - 'common': ['get_available_updates', 'get_migration_scripts',
 #               'set_loglevel']
 _cause_message = ("\nThe above exception was the direct cause "
@@ -323,15 +317,15 @@ class Service(object):
             proxy = server.netsvc.ExportService.getService(endpoint)
             self._dispatch = proxy.dispatch
         self._endpoint = endpoint
-        self._methods = sorted(methods)
+        self._methods = methods
         self._verbose = verbose
 
     def __repr__(self):
-        return '<Service %r>' % (self._rpcpath + self._endpoint)
+        return "<Service '%s%s'>" % (self._rpcpath, self._endpoint)
     __str__ = __repr__
 
     def __dir__(self):
-        return self._methods
+        return sorted(self._methods)
 
     def __getattr__(self, name):
         if name not in self._methods:
@@ -370,7 +364,7 @@ class Client(object):
     This is the top level object.
     The `server` is the URL of the instance, like ``http://localhost:8069``.
     If `server` is an ``openerp`` module, it is used to connect to the local
-    server (6.1 only).
+    server (>= 6.1).
 
     The `db` is the name of the database and the `user` should exist in the
     table ``res.users``.  If the `password` is not provided, it will be
@@ -391,7 +385,7 @@ class Client(object):
         major_version = None
 
         def get_proxy(name):
-            if major_version in ('5.0', None):
+            if major_version in ('5.0', None) or name == 'wizard':
                 methods = _methods[name]
             else:
                 # Only for OpenERP >= 6
@@ -635,15 +629,13 @@ class Client(object):
         for item in kwargs.items():
             print('Ignoring: %s = %r' % item)
         res = self._execute(obj, method, *params)
-        if res and ordered:
+        if ordered:
             # The results are not in the same order as the ids
             # when received from the server
-            assert len(res) == len(set(ids))
             resdic = dict([(val['id'], val) for val in res])
-            if isinstance(ordered, list):
-                res = [id_ and resdic[id_] for id_ in ordered]
-            else:
-                res = [resdic[id_] for id_ in ids]
+            if not isinstance(ordered, list):
+                ordered = ids
+            res = [resdic.get(id_, False) for id_ in ordered]
         return res
 
     def exec_workflow(self, obj, signal, obj_id):
@@ -819,13 +811,14 @@ class Client(object):
         names = [m['model'] for m in models]
         return dict([(mixedcase(name), self._model(name)) for name in names])
 
-    def model(self, name):
+    def model(self, name, check=True):
         """Return a :class:`Model` instance.
 
-        The argument `name` is the name of the model.
+        The argument `name` is the name of the model.  If the optional
+        argument `check` is :const:`False`, no validity check is done.
         """
         try:
-            return self._models[name]
+            return self._models[name] if check else self._model(name)
         except KeyError:
             models = self.models(name)
         if name in self._models:
@@ -887,8 +880,8 @@ class Client(object):
             self.__dict__[method] = rv
             return rv
         if method.startswith('_'):
-            raise AttributeError("'Client' object has no attribute %r" %
-                                 method)
+            errmsg = "'Client' object has no attribute %r" % method
+            raise AttributeError(errmsg)
 
         # miscellaneous object methods
         def wrapper(self, obj, *params, **kwargs):
@@ -1017,14 +1010,15 @@ class Model(object):
             field_type = field['type']
             if field_type == 'many2one':
                 if value:
-                    rel_model = self.client.model(field['relation'])
+                    rel_model = self.client.model(field['relation'], False)
                     values[key] = Record(rel_model, value, context=context)
             elif field_type in ('one2many', 'many2many'):
-                rel_model = self.client.model(field['relation'])
+                rel_model = self.client.model(field['relation'], False)
                 values[key] = RecordList(rel_model, value, context=context)
             elif value and field_type == 'reference':
                 res_model, res_id = value.split(',')
-                values[key] = Record(self.client.model(res_model), int(res_id))
+                rel_model = self.client.model(res_model, False)
+                values[key] = Record(rel_model, int(res_id))
         return values
 
     def _unbrowse_values(self, values):
@@ -1040,7 +1034,7 @@ class Model(object):
             if field_type in ('one2many', 'many2many'):
                 if not value:
                     new_values[key] = [(6, 0, [])]
-                elif isinstance(value[0], (int, long)):
+                elif isinstance(value[0], int_types):
                     new_values[key] = [(6, 0, value)]
         return new_values
 
@@ -1122,17 +1116,17 @@ class RecordList(object):
             field = self._model._fields.get(fields)
             if field:
                 if field['type'] == 'many2one':
-                    rel_model = client.model(field['relation'])
+                    rel_model = client.model(field['relation'], False)
                     return RecordList(rel_model, values, context=context)
                 if field['type'] in ('one2many', 'many2many'):
-                    rel_model = client.model(field['relation'])
+                    rel_model = client.model(field['relation'], False)
                     return [RecordList(rel_model, v) for v in values]
                 if field['type'] == 'reference':
                     records = []
                     for value in values:
                         if value:
                             res_model, res_id = value.split(',')
-                            rel_model = client.model(res_model)
+                            rel_model = client.model(res_model, False)
                             value = Record(rel_model, int(res_id))
                         records.append(value)
                     return records
@@ -1171,11 +1165,6 @@ class RecordList(object):
         if attr.startswith('_'):
             errmsg = "'RecordList' object has no attribute %r" % attr
             raise AttributeError(errmsg)
-        if attr == '_ids':
-            # deprecated since 1.2.1
-            warnings.warn("Attribute 'RecordList._ids' is deprecated, "
-                          "use 'RecordList.id' instead.")
-            return self.id
 
         def wrapper(self, *params, **kwargs):
             """Wrapper for client.execute(%r, %r, [...], *params, **kwargs)."""
@@ -1331,11 +1320,6 @@ class Record(object):
             return name
         if attr.startswith('_'):
             raise AttributeError("'Record' object has no attribute %r" % attr)
-        if attr == 'client':
-            # deprecated since 1.2.1
-            warnings.warn("Attribute 'Record.client' is deprecated, "
-                          "use 'Record._model.client' instead.")
-            return self._model.client
 
         def wrapper(self, *params, **kwargs):
             """Wrapper for client.execute(%r, %r, %d, *params, **kwargs)."""
